@@ -12,6 +12,7 @@ from typing import (
     Protocol,
     runtime_checkable,
     Callable,
+    overload,
 )
 
 Callback = Callable[..., Any]
@@ -118,6 +119,17 @@ class Camera(BaseCamera):
 
 
 # camera types
+CameraExtension = Literal[
+    "hub",
+    "prebuffer",
+    "motionDetection",
+    "objectDetection",
+    "audioDetection",
+    "ptz",
+    "intercom",
+]
+
+
 class BaseCameraConfig(TypedDict, total=False):
     name: str
     nativeId: Optional[str]
@@ -408,6 +420,7 @@ class PluginConfigService(Protocol):
 
 
 # manager todo: improve typing
+@runtime_checkable
 class SystemManager(Protocol):
     def on(self, event: str, listener: Callable) -> "SystemManager": ...
     def once(self, event: str, listener: Callable) -> "SystemManager": ...
@@ -415,6 +428,7 @@ class SystemManager(Protocol):
     def remove_all_listeners(self, event: str) -> "SystemManager": ...
 
 
+@runtime_checkable
 class PluginsManager(Protocol):  # todo: add methods
     def on(self, event: str, listener: Callable) -> "PluginsManager": ...
     def once(self, event: str, listener: Callable) -> "PluginsManager": ...
@@ -422,15 +436,99 @@ class PluginsManager(Protocol):  # todo: add methods
     def remove_all_listeners(self, event: str) -> "PluginsManager": ...
 
 
+@runtime_checkable
+class CameraDevice(Protocol):  # todo
+    pass
+
+
+# device manager
+DeviceManagerEventType = Literal[
+    "cameraSelected",
+    "cameraDeselected",
+    "cameraAdded",
+    "cameraRemoved",
+    "cameraUpdated",
+]
+
+CameraSelectedCallback = Callable[[CameraDevice, CameraExtension], None]
+CameraDeselectedCallback = Callable[[str, CameraExtension], None]
+CameraAddedCallback = Callable[[CameraDevice], None]
+CameraRemovedCallback = Callable[[str], None]
+CameraUpdatedCallback = Callable[[str, CameraDevice], None]
+
+
+@runtime_checkable
 class DeviceManager(Protocol):  # todo: add methods
-    def on(self, event: str, listener: Callable) -> "DeviceManager": ...
-    def once(self, event: str, listener: Callable) -> "DeviceManager": ...
-    def remove_listener(self, event: str, listener: Callable) -> "DeviceManager": ...
-    def remove_all_listeners(self, event: str) -> "DeviceManager": ...
+    @overload
+    def on(
+        self, event: Literal["cameraSelected"], listener: CameraSelectedCallback
+    ) -> "DeviceManager": ...
+    @overload
+    def on(
+        self, event: Literal["cameraDeselected"], listener: CameraDeselectedCallback
+    ) -> "DeviceManager": ...
+    @overload
+    def on(
+        self, event: Literal["cameraAdded"], listener: CameraAddedCallback
+    ) -> "DeviceManager": ...
+    @overload
+    def on(
+        self, event: Literal["cameraRemoved"], listener: CameraRemovedCallback
+    ) -> "DeviceManager": ...
+    @overload
+    def on(
+        self, event: Literal["cameraUpdated"], listener: CameraUpdatedCallback
+    ) -> "DeviceManager": ...
+
+    @overload
+    def once(
+        self, event: Literal["cameraSelected"], listener: CameraSelectedCallback
+    ) -> "DeviceManager": ...
+    @overload
+    def once(
+        self, event: Literal["cameraDeselected"], listener: CameraDeselectedCallback
+    ) -> "DeviceManager": ...
+    @overload
+    def once(
+        self, event: Literal["cameraAdded"], listener: CameraAddedCallback
+    ) -> "DeviceManager": ...
+    @overload
+    def once(
+        self, event: Literal["cameraRemoved"], listener: CameraRemovedCallback
+    ) -> "DeviceManager": ...
+    @overload
+    def once(
+        self, event: Literal["cameraUpdated"], listener: CameraUpdatedCallback
+    ) -> "DeviceManager": ...
+
+    @overload
+    def remove_listener(
+        self, event: Literal["cameraSelected"], listener: CameraSelectedCallback
+    ) -> "DeviceManager": ...
+    @overload
+    def remove_listener(
+        self, event: Literal["cameraDeselected"], listener: CameraDeselectedCallback
+    ) -> "DeviceManager": ...
+    @overload
+    def remove_listener(
+        self, event: Literal["cameraAdded"], listener: CameraAddedCallback
+    ) -> "DeviceManager": ...
+    @overload
+    def remove_listener(
+        self, event: Literal["cameraRemoved"], listener: CameraRemovedCallback
+    ) -> "DeviceManager": ...
+    @overload
+    def remove_listener(
+        self, event: Literal["cameraUpdated"], listener: CameraUpdatedCallback
+    ) -> "DeviceManager": ...
+
+    def remove_all_listeners(
+        self, event: Optional[DeviceManagerEventType] = None
+    ) -> "DeviceManager": ...
 
 
 # plugin api
-EventType = Literal["finishLaunching", "shutdown"]
+APIEventType = Literal["finishLaunching", "shutdown"]
 
 
 @runtime_checkable
@@ -443,19 +541,18 @@ class PluginAPI(Protocol):
     system_manager: SystemManager
     plugins_manager: PluginsManager
 
-    def on(self, event: EventType, listener: Callable) -> "PluginAPI": ...
-    def once(self, event: EventType, listener: Callable) -> "PluginAPI": ...
-    def remove_listener(self, event: EventType, listener: Callable) -> "PluginAPI": ...
-    def remove_all_listeners(self, event: EventType) -> "PluginAPI": ...
+    def on(self, event: APIEventType, listener: Callable) -> "PluginAPI": ...
+    def once(self, event: APIEventType, listener: Callable) -> "PluginAPI": ...
+    def remove_listener(
+        self, event: APIEventType, listener: Callable
+    ) -> "PluginAPI": ...
+    def remove_all_listeners(
+        self, event: Optional[APIEventType] = None
+    ) -> "PluginAPI": ...
 
 
 # base plugin
 class BasePlugin(ABC):
-    def __init__(self, plugin_id: str, log: PluginLogger, api: PluginAPI) -> None:
-        self.id: str = plugin_id
-        self.api: PluginAPI = api
-        self.logger: PluginLogger = log
-
     @abstractmethod
     async def on_form_submit(self, action_id: str, payload: Any) -> Optional[dict]:
         pass
